--Mod by ttlala
rotatingParts = {};

function rotatingParts.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Motorized, specializations);
end;

function rotatingParts:load(xmlFile)
    self.rotatingParts = {};
    local rotatingPartsCount = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.rotatingParts#count"), 0);	
	for i=1, rotatingPartsCount do
		local namei = string.format("vehicle.rotatingParts.rotatingPart%d", i);
		local rotatingPart = Utils.indexToObject(self.rootNode, getXMLString(xmlFile, namei .. "#index"));
		
		self.rotatingParts[i] = {};
		self.rotatingParts[i].node = rotatingPart;
		
        local x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, namei .. "#minRot"));
        self.rotatingParts[i].minRot = {};
        self.rotatingParts[i].minRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotatingParts[i].minRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotatingParts[i].minRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		
		x, y, z = Utils.getVectorFromString(getXMLString(xmlFile, namei .. "#maxRot"));
		self.rotatingParts[i].maxRot = {};
        self.rotatingParts[i].maxRot[1] = Utils.degToRad(Utils.getNoNil(x, 0));
        self.rotatingParts[i].maxRot[2] = Utils.degToRad(Utils.getNoNil(y, 0));
        self.rotatingParts[i].maxRot[3] = Utils.degToRad(Utils.getNoNil(z, 0));
		
		self.rotatingParts[i].rotTime = Utils.getNoNil(getXMLString(xmlFile, namei .. "#rotTime"), 2)*1000;
	end;
	
	self.isLeftMirror = 0;
	self.isRightMirror = 0;
	self.isLeftMirrorUD = 0;
	self.isRightMirrorUD = 0;
	self.rotationUD = false; 
end;

function rotatingParts:delete()
end;

function rotatingParts:mouseEvent(posX, posY, isDown, isUp, button)
end;

function rotatingParts:keyEvent(unicode, sym, modifier, isDown)
    if sym == Input.KEY_KP_4 or sym == Input.KEY_KP_7 then
        self.isLeftMirror = 0;
		self.isLeftMirrorUD = 0;
        if isDown then
        	if sym == Input.KEY_KP_7 then
                self.isLeftMirror = 1;
				self.isLeftMirrorUD = -1;
        	elseif sym == Input.KEY_KP_4 then
                self.isLeftMirror = -1;
				self.isLeftMirrorUD = 1;
        	end;
        end;
	end;
    if sym == Input.KEY_KP_6 or sym == Input.KEY_KP_9 then
        self.isRightMirror = 0;
		self.isRightMirrorUD = 0;
        if isDown then
        	if sym == Input.KEY_KP_9 then
                self.isRightMirror = 1;
				self.isRightMirrorUD = -1;
        	elseif sym == Input.KEY_KP_6 then
                self.isRightMirror = -1;
				self.isRightMirrorUD = 1;
        	end;
        end;
	end;
end;

function rotatingParts:update(dt)	
    local jointFrameInvalid = false;
    if self.isEntered then
		if InputBinding.hasEvent(InputBinding.UDMIRR) then
			self.rotationUD = not self.rotationUD;
		end;
		
		--if InputBinding.hasEvent(InputBinding.RIGHTMIRR) then	
			--self.isRightMirror = not self.isRightMirror;
		--end;
		
		-- Rotate Left Mirror
		if self.rotationUD==false then
			if self.rotatingParts[1] ~= nil then
				if self.isLeftMirror ~=  0 then -- Z tengely
					local x, y, z = getRotation(self.rotatingParts[1].node);
					local x1, y1, z1 = getRotation(self.rotatingParts[3].node);
					local rot = {x,y1,z1};
					local minRot = {};
					local maxRot = {};
					minRot[1] = x1;
					minRot[2] = y1;
					minRot[3] = self.rotatingParts[1].minRot[3];
					maxRot[1] = x1;
					maxRot[2] = y1;
					maxRot[3] = self.rotatingParts[1].maxRot[3];
					if minRot[3] <  maxRot[3] then
						maxRot[3] = maxRot[3] * -1
					end;
					--renderText(0.7543, 0.2540, 0.02, z ..";" .. y1 ..";" .. x1 ); --X,Y,vastag,mit
					--renderText(0.1543, 0.4540, 0.02, "z:" .. z)
					--renderText(0.1543, 0.3540, 0.02, "x1:" .. x1)
					--renderText(0.1543, 0.2540, 0.02, "y:" .. y1)
					--renderText(0.1543, 0.1540, 0.02, "self.isLeftMirror:" .. self.isLeftMirror)
					local newRot = Utils.getMovedLimitedValues(rot, maxRot, minRot, 3, self.rotatingParts[1].rotTime, dt, self.isLeftMirror == 1);
					--renderText(0.1543, 0.4540, 0.02, "newRot:" .. unpack(newRot))
					setRotation(self.rotatingParts[1].node, unpack(newRot));
					for i=1, 3 do
						if math.abs(newRot[i] - rot[i]) > 0.001 then
							jointFrameInvalid = true;
						end;
					end;
				------------					
				--	local x, y, z = getRotation(self.rotatingParts[1].node);
				--	local rot = {x,y,z};
				--	renderText(0.7543, 0.2540, 0.02, z ..";" .. y ..";" .. x ); --X,Y,vastag,mit
				--	local newRot = Utils.getMovedLimitedValues(rot, self.rotatingParts[1].maxRot, self.rotatingParts[1].minRot, 3, self.rotatingParts[1].rotTime, dt, self.isLeftMirror==1);
				--	setRotation(self.rotatingParts[1].node, unpack(newRot));
				--	for i=1, 3 do
				--		if math.abs(newRot[i] - rot[i]) > 0.001 then
				--			jointFrameInvalid = true;
				--		end;
				--	end;
					local x1, y1, z1 = getRotation(self.rotatingParts[1].node);
				end;   
			end;
			-- Rotate Right Mirror
			if self.rotatingParts[2] ~= nil then 
				if self.isRightMirror ~=  0 then -- Z tengely
					local x, y, z = getRotation(self.rotatingParts[2].node);
					local x1, y1, z1 = getRotation(self.rotatingParts[4].node);
					local rot = {x,y1,z1};
					local minRot = {};
					local maxRot = {};
					minRot[1] = x1;
					minRot[2] = y1;
					minRot[3] = self.rotatingParts[2].minRot[3];
					maxRot[1] = x1;
					maxRot[2] = y1;
					maxRot[3] = self.rotatingParts[2].maxRot[3];
					if minRot[3] >  maxRot[3] then
						maxRot[3] = maxRot[3] * -1
					end;
					--renderText(0.7543, 0.2540, 0.02, z ..";" .. y1 ..";" .. x1 ); --X,Y,vastag,mit
					local newRot = Utils.getMovedLimitedValues(rot, maxRot, minRot, 3, self.rotatingParts[2].rotTime, dt, self.isRightMirror == 1);
					setRotation(self.rotatingParts[2].node, unpack(newRot));
					for i=1, 3 do
						if math.abs(newRot[i] - rot[i]) > 0.001 then
							jointFrameInvalid = true;
						end;
					end;
				------------
				--	local x, y, z = getRotation(self.rotatingParts[2].node);
				--	local rot = {x,y,z};
				--	renderText(0.7543, 0.2540, 0.02, z ..";" .. y ..";" .. x ); --X,Y,vastag,mit
				--	local newRot = Utils.getMovedLimitedValues(rot, self.rotatingParts[2].maxRot, self.rotatingParts[2].minRot, 3, self.rotatingParts[2].rotTime, dt, self.isRightMirror==1);
				--	setRotation(self.rotatingParts[2].node, unpack(newRot));
				--	for i=1, 3 do
				--		if math.abs(newRot[i] - rot[i]) > 0.001 then
				--			jointFrameInvalid = true;
				--		end;
				--	end;
				local x2, y2, z2 = getRotation(self.rotatingParts[2].node);
				end;
			end;
		else
			if self.rotatingParts[3] ~= nil then
				if self.isLeftMirrorUD ~=  0 then -- X tengely
		----
		--local x,y,z = getRotation(self.rotatingParts[3].node);
		--if self.isLeftMirrorUD == 1 then
		--x = x + 0.01;
		--self.jointmove = true;
		--elseif self.isLeftMirrorUD == -1 then
		--x = x - 0.01;
		--self.jointmove = true;
		--end;
		--setRotation(self.rotatingParts[3], x,y,z);
		--renderText(0.7543, 0.2540, 0.02, x ..";" .. y ..";" .. y ); --X,Y,vastag,mit
		--self.jointmove = false;
		----
					local x, y, z = getRotation(self.rotatingParts[3].node);
					local x1, y1, z1 = getRotation(self.rotatingParts[1].node);
					local rot = {x,y1,z1};
					local minRot = {};
					local maxRot = {};
					minRot[1] = self.rotatingParts[3].minRot[1];
					minRot[2] = y1;
					minRot[3] = z1;
					maxRot[1] = self.rotatingParts[3].maxRot[1];
					maxRot[2] = y1;
					maxRot[3] = z1;
					if minRot[1] >  maxRot[1] then
						maxRot[1] = maxRot[1] * -1
					end;
					--renderText(0.7543, 0.2540, 0.02, x ..";" .. y1 ..";" .. z1 ); --X,Y,vastag,mit
					local newRot = Utils.getMovedLimitedValues(rot, maxRot, minRot, 3, self.rotatingParts[3].rotTime, dt, self.isLeftMirrorUD == 1);
					setRotation(self.rotatingParts[3].node, unpack(newRot));
					for i=1, 3 do
						if math.abs(newRot[i] - rot[i]) > 0.001 then
							jointFrameInvalid = true;
						end;
					end;
				end;   
			end;
			-- Rotate Right Mirror
			if self.rotatingParts[4] ~= nil then
				if self.isRightMirrorUD ~=  0 then -- X tengely
		----
		--local x,y,z = getRotation(self.rotatingParts[3].node);
		--if self.isRightMirrorUD == 1 then
		--x = x + 0.01;
		--elseif self.isRightMirrorUD == -1 then
		--x = x - 0.01;
		--end;
		--setRotation(self.rotatingParts[4], x,y,z);
		--renderText(0.7543, 0.2540, 0.02, x ..";" .. y ..";" .. y ); --X,Y,vastag,mit
		--self.jointmove = false;
		----
					local x, y, z = getRotation(self.rotatingParts[4].node);
					local x1, y1, z1 = getRotation(self.rotatingParts[2].node);
					local rot = {x,y1,z1};
					local minRot = {};
					local maxRot = {};
					minRot[1] = self.rotatingParts[4].minRot[1];
					minRot[2] = y1;
					minRot[3] = z1;
					maxRot[1] = self.rotatingParts[4].maxRot[1];
					maxRot[2] = y1;
					maxRot[3] = z1;
					if minRot[1] >  maxRot[1] then
						maxRot[1] = maxRot[1] * -1
					end;
					--renderText(0.7543, 0.2540, 0.02, x ..";" .. y1 ..";" .. z1 ); --X,Y,vastag,mit
					local newRot = Utils.getMovedLimitedValues(rot, maxRot, minRot, 3, self.rotatingParts[4].rotTime, dt, self.isRightMirrorUD == 1);
					setRotation(self.rotatingParts[4].node, unpack(newRot));
					for i=1, 3 do
						if math.abs(newRot[i] - rot[i]) > 0.001 then
							jointFrameInvalid = true;
						end;
					end;
				end;
			end;
		end;
	end;
end;

function rotatingParts:draw()
	g_currentMission:addExtraPrintText(string.format(g_i18n:getText("LMirrors_controls"), self.typeDesc) .. " Up-left/Down-right:Num4/7 Switch:Num8");
	g_currentMission:addExtraPrintText(string.format(g_i18n:getText("RMirrors_controls"), self.typeDesc) .. " Up-left/Down-right:Num6/9 Switch:Num8");
	--g_currentMission:addHelpButtonText(g_i18n:getText("MirrorLeft"), InputBinding.MRight);
	--g_currentMission:addHelpButtonText(g_i18n:getText("MirrorRight"), InputBinding.MLeft);
	--g_currentMission:addHelpButtonText(g_i18n:getText("MirrorSwitch"), InputBinding.UDMIRR);
end;
